<!DOCTYPE html>
<html>
  <head>
    <title>Guestbook Demo</title>
  </head>
  <body>
    <p>
      {{with .Email}}You are currently logged in as {{.}}.{{end}}
      {{with .Login}}<a href="{{.}}">Sign in</a>{{end}}
      {{with .Logout}}<a href="{{.}}">Sign out</a>{{end}}
    </p>

    {{range .Greetings }}
    <p>
      {{with .Author}}<b>{{.}}</b>{{else}}An anonymous person{{end}}
      on <em>{{.Date.Format "3:04pm, Mon 2 Jan"}}</em>
      wrote <blockquote>{{.Content}}</blockquote>
    </p>
    {{end}}

    <form action="/sign" method="post">
      <div><textarea name="content" rows="3" cols="60"></textarea></div>
      <div><input type="submit" value="Sign Guestbook"></div>
    </form>
  </body>
</html>
